/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <linux/reboot.h>

#include <sys/reboot.h>
#include <sys/syscall.h>

typedef enum {
	ACTION_REBOOT,
	ACTION_HALT,
	ACTION_POWEROFF
} action;

enum {
	OPT_HELP = 0x100,
	OPT_HALT,
	OPT_REBOOT,
	OPT_NO_WALL
};

static struct option long_options[] = {
	{ "force",	no_argument,	NULL,	'f'	    },
	{ "halt",	no_argument,	NULL,	OPT_HALT    },
	{ "help",	no_argument,	NULL,	OPT_HELP    },
	{ "no-sync",	no_argument,	NULL,	'n'	    },
	{ "no-wall",	no_argument,	NULL,	OPT_NO_WALL },
	{ "no-wtmp",	no_argument,	NULL,	'd'	    },
	{ "poweroff",	no_argument,	NULL,	'p'	    },
	{ "reboot",	no_argument,	NULL,	OPT_REBOOT  },
	{ NULL,			  0,	   0,	 0	    }
};

static const char *exe = NULL;

/* ------------------------------------------------------------------------ */

	void
usage(void)
{
	printf("Usage: %s [OPTION...] [ARG]\n"
			"  -d --no-wtmp   Ignored in this version\n"
			"  -f --force     Ignored in this version\n"
			"  -n --no-sync   Ignored in this version\n"
			"  -p --poweroff  Switch off the device\n"
			"     --reboot    Reboot the device\n"
			"     --halt      Halt the device\n"
			"     --no-wall   Ignored in this version\n"
			"     --help      Display this help and exit\n", exe);
}

/* ------------------------------------------------------------------------ */

	static int
reboot_or_poweroff(action mode, const char *cmd)
{
	sync();
	if (mode == ACTION_REBOOT) {
		if (cmd) {
			printf("Rebooting with command \"%s\".", cmd);
			syscall(__NR_reboot, LINUX_REBOOT_MAGIC1,
					LINUX_REBOOT_MAGIC2,
					LINUX_REBOOT_CMD_RESTART2, cmd);
			perror("syscall(SYS_reboot)");
		}

		printf("Rebooting.\n");
		reboot(RB_AUTOBOOT);
	} else if (mode == ACTION_HALT) {
		printf("Halting.\n");
		reboot(RB_HALT_SYSTEM);
	} else {
		printf("Powering off.\n");
		reboot(RB_POWER_OFF);
	}

	perror("reboot()");
	return 1;
}

/* ------------------------------------------------------------------------ */

	int
main(int argc, char *argv[])
{
	int c;
	char *cmd = NULL;
	action mode = ACTION_REBOOT;

	exe = strrchr(argv[0], '/') ? strrchr(argv[0], '/') + 1 : argv[0];
	opterr = 0;
	while ((c = getopt_long(argc, argv, "dfnp", long_options,
					NULL)) >= 0) {
		switch (c) {
		case OPT_HALT:
			mode = ACTION_HALT;
			break;
		case OPT_REBOOT:
			mode = ACTION_REBOOT;
			break;
		case 'p':
			mode = ACTION_POWEROFF;
			break;
		case 'd':
		case 'f':
		case 'n':
		case OPT_NO_WALL:
			break; /* Ignore option */
		case OPT_HELP:
			usage();
			exit(0);
		default:
			usage();
			exit(1);
		}
	}

	if (mode == ACTION_REBOOT && argc == optind + 1)
		cmd = argv[optind];
	else if (optind < argc) {
		printf("Too many arguments.\n");
		usage();
		exit(1);
	}

	return reboot_or_poweroff(mode, cmd);
}
